SystemOrganization addCategory: #'PVpub-AnalogClockPrototype06'!

CircleMorph subclass: #Dial06
	instanceVariableNames: 'boundsLag watchhands watchhand06sec watchhand06min watchhand06hour'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClockPrototype06'!

!Dial06 commentStamp: 'pv 1/2/2014 15:46' prior: 0!
* Introduction to AnalogClock
This is the project manual for the AnalogClock tutorial.

It has been written in Org. This is an emacs-mode which helps organizing projects.

You can fetch the sources from [http://ss3.gemstone.com/ss/AnalogClock SqueakSource-AnalogClock]
Before loading into your image you should create two classes:
- Graphics-Primitives/LayoutConcentricFrame
- Morphic-Layouts/ConcentricLayout.
The sources come with the implementation of these two classes and 
small extensions for: 
- MorphicKernel/Morph>>layoutConcentricallyIn:, 
- Morphic-Layouts/LayoutPolicy>>isConcentricLayout.

If you want to start AnalogClock, open a workspace and execute
#+NAME: Compute Instance-LOC and Class-LOC
#+BEGIN_SRC Smalltalk
Dial05 new openInWorld.
#+END_SRC 

This project manual comes with the first six chapters built along a symmetry axis.
The first three chapters treat the translation steps from Product Requirements
to Architecture and Design and to Implementation. The next three chapters
comment the verification actions taken for the intermediate products of 
chapters one to three.
* Product Requirements
** Functional
*** Show an analog clock in Squeak
Show an analog clock with hour and minute watchhand.
:PROPERTIES:
:CUSTOM_ID: REQ101
:END:
*** Show a rotating watchhand
Show a PolygonMorph which rotates.
:PROPERTIES:
:CUSTOM_ID: REQ106
:END:
*** Show a rotating watchhand on a dial
Show a PolygonMorph which rotates on a dial formed like a circle.
:PROPERTIES:
:CUSTOM_ID: REQ107
:END:
*** Show a static watchhand on a dial able to get resized
Show a PolygonMorph on a dial which adapts when the dial gets resized.
:PROPERTIES:
:CUSTOM_ID: REQ108
:END:
*** Show a rotating watchhand on a dial able to get resized
Show a PolygonMorph rotating on a dial which adapts when the dial gets resized.
:PROPERTIES:
:CUSTOM_ID: REQ109
:END:
*** Synchronize clock to the OS time
The time shown on the AnalogClock is the same as the time of the OS we are running on.
:PROPERTIES:
:CUSTOM_ID: REQ102
:END:
*** Respond to the Pick up-Halo
The AnalogClock may be moved around. All its submorphs are included in an adequate manner.
:PROPERTIES:
:CUSTOM_ID: REQ103
:END:
*** Respond to the Resize-Halo
The AnalogClock may get resized. All its submorphs are included in an adequate manner.
:PROPERTIES:
:CUSTOM_ID: REQ104
:END:
*** Be able to embed one AnalogClock in another
The AnalogClock may get embedded into another one.
:PROPERTIES:
:CUSTOM_ID: REQ105
:END:
*** Do not show elements which are too small to recognize them
When resizing makes the clock get too small, do not show the watchhand anymore.
:PROPERTIES:
:CUSTOM_ID: REQ112
:END:
*** Show three watchhands: secons, minutes and hours
When resizing makes the clock get too small, do not show the watchhand anymore.
:PROPERTIES:
:CUSTOM_ID: REQ113
:END:
** Non-Functional
*** Store project information in Squeak
Show information about the application lifecycle in the class comment. Use org-mode-format.
:PROPERTIES:
:CUSTOM_ID: REQ110
:END:
** Process
*** Write unit tests
:PROPERTIES:
:CUSTOM_ID: REQ111
:END:
** Tools
* Architecture and Design
** abc
*** Design Decision: Avoid doing everything in drawOn:
[http://www.squeaksource.com/WatchMorph WatchMorph] relies on drawOn:. With each time step and on each pich up- or resize-operation the whole watch is drawn.
This seems ok for a simple watch. But it is not scalable.
Decision <2013-11-24 Sun>: adapt Morphic's Layout-mechanism (TableLayout).
Traces to: [[#REQ108]]
:PROPERTIES:
:CUSTOM_ID: DES001
:END:
*** Design Decision: Hide implementation details of two watchhands for resize-operation
We use two watchhands to implement correct resize behaviour. The interface for a Dial should get concentrated in one class.
Decision <2013-12-02 Mon>: create ResizableWatchhand.
Traces to: [[#REQ109]]
:PROPERTIES:
:CUSTOM_ID: DES002
:END:
*** Design Decision: Create a control interface for the clock
Create a class which acts like an interface for a user control panel and for tests.
Decision <2013-12-04 Tue>: create Dial05Interface.
Traces to: 
:PROPERTIES:
:CUSTOM_ID: DES003
:END:
*** Design Decision: Hide implementation details of two watchhands for resize-operation by using 2 objects instead of 3
We use two watchhands to implement correct resize behaviour. The interface for a Dial should get concentrated in one class.
Use one additional object to implement the midnight watchhand.
Decision <2013-12-27 Tue>: create Watchhand06 (visible rotating watchhand) and its member variable watchhand06Midnight (invisible static watchhand).
Traces to: 
:PROPERTIES:
:CUSTOM_ID: DES004
:END:
* Implementation
** Step 1
*** PVpub-AnalogClockPrototype01 (WatchHand01)
Shows a rotating PolygonMorph.
:PROPERTIES:
:CUSTOM_ID: IMP001
:END:
** Step 2
*** PVpub-AnalogClockPrototype02 (Dial02, WatchHand02)
Shows a rotating watchhand (PolygonMorph) rotating on a dial (CircleMorph). When resizing the clock, the watchhand does not adapt.
:PROPERTIES:
:CUSTOM_ID: IMP002
:END:
** Step 3
*** PVpub-AnalogClockPrototype03 (Dial03, WatchHand03)
Shows a non-rotating watchhand (PolygonMorph) on a dial (CircleMorph). When resizing the clock, the watchhand does adapt.
:PROPERTIES:
:CUSTOM_ID: IMP003
:END:
** Step 4
*** PVpub-AnalogClockPrototype04 (Dial04, WatchHand04)
Shows a rotating watchhand (PolygonMorph) on a dial (CircleMorph). When resizing the clock, the watchhand does adapt.
:PROPERTIES:
:CUSTOM_ID: IMP004
:END:
** Step 5
*** PVpub-AnalogClockPrototype05 (Dial05, ResizableWatchHand05)
Shows a rotating watchhand (PolygonMorph) on a dial (CircleMorph). When resizing the clock, the watchhand does adapt. The interplay between Dial and WatchHand is simplified. An analog clock may get embedded into another.
:PROPERTIES:
:CUSTOM_ID: IMP005
:END:
** Step 6
*** PVpub-AnalogClockPrototype06 (Dial06, Watchhand06)
Shows a rotating watchhand (Watchhand06) on a dial (CircleMorph). When resizing the clock, the watchhand does adapt. Compared to Step 5 the design is simplified.
:PROPERTIES:
:CUSTOM_ID: IMP006
:END:
* Verification of Implementation
* Verification at Architecture Level
* Verification at Product Level
** Versions and Releases
*** PVpub-AnalogClockPrototype01
**** VER001
abc
***** Implemented Requirements
[[#REQ106]]
***** Metrics
:PROPERTIES:
:SWA-LINT_ClassNotReferenced: 3
:INSTANCE-LOC:                11
:CLASS-LOC:                   0
:MCZ:                         PVpub-AnalogClockPrototype01-pv.1
:END:
*** PVpub-AnalogClockPrototype02
**** VER001
abc
***** Implemented Requirements
[[#REQ107]]
***** Metrics
:PROPERTIES:
:SWA-LINT_ClassNotReferenced: 1
:SWA-Lint_InconsistentMethodClassification: 1
:INSTANCE-LOC:                35
:CLASS-LOC:                   0
:MCZ:                         PVpub-AnalogClockPrototype02-pv.1
:END:
*** PVpub-AnalogClockPrototype03
**** VER001
abc
***** Implemented Requirements
[[#REQ108]]
***** Metrics
:PROPERTIES:
:SWA-LINT_ClassNotReferenced: 1
:SWA-Lint_InconsistentMethodClassification: 1
:INSTANCE-LOC:                32
:CLASS-LOC:                   0
:MCZ:                         PVpub-AnalogClockPrototype03-pv.1
:END:
*** PVpub-AnalogClockPrototype04
**** VER001
abc
***** Implemented Requirements
[[#REQ109]]
***** Metrics
:PROPERTIES:
:CUSTOM_ID:                   VER-004-001
:SWA-LINT_ClassNotReferenced: 2
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                84
:CLASS-LOC:                   6
:MCZ:                         PVpub-AnalogClockPrototype04-pv.1
:END:
*** PVpub-AnalogClockPrototype05
**** VER001
abc
***** Implemented Requirements
[[#REQ109]]
***** Implemented Design Decision
[[#DES002]]
***** Metrics
:PROPERTIES:
:SWA-LINT_ClassNotReferenced: 3
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                108
:CLASS-LOC:                   6
:MCZ:                         PVpub-AnalogClockPrototype05-pv.1
:END:
**** VER002
Add class comment (org-mode)
***** Implemented Requirements
[[#REQ110]]
***** Metrics
:PROPERTIES:
:SWA-LINT_ClassNotReferenced: 3
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                108
:CLASS-LOC:                   6
:MCZ:                         PVpub-AnalogClockPrototype05-pv.2
:MCZ-TIME:                    Time: 2 December 2013, 10:33:09.286 pm
:END:
**** VER003
renaming
***** Implemented Requirements
none
***** Metrics
:PROPERTIES:
:SWA-LINT_ClassNotReferenced: 3
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                112
:CLASS-LOC:                   11
:MCZ:                         PVpub-AnalogClockPrototype05-pv.3
:MCZ-TIME:                    Time: 4 December 2013, 10:38:15.566 pm
:END:
**** VER004
Control interface
***** Implemented Requirements
implement [[#DES003]]
***** Metrics
:PROPERTIES:
:SWA-LINT_MethodsImplementedButNotSent: 2
:SWA-LINT_VariableReferencedInOnlyOneMethod: 2
:SWA-LINT_ClassNotReferenced: 5
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                122
:CLASS-LOC:                   11
:MCZ:                         PVpub-AnalogClockPrototype05-pv.4
:MCZ-TIME:                    Time: 4 December 2013, 11:15:51.485 pm
:END:
**** VER005
Add unit tests
***** Implemented Requirements
implement [[#REQ111]]
***** Metrics PVpub-AnalogClockPrototype05
:PROPERTIES:
:SWA-LINT_MethodsImplementedButNotSent: 1
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                139
:CLASS-LOC:                   11
:MCZ:                         PVpub-AnalogClockPrototype05-pv.5
:MCZ-TIME:                    Time: 5 December 2013, 9:41:27.372 pm
:END:
***** Metrics PVpubTests-AnalogClockPrototype05
:PROPERTIES:
:SWA-LINT_MethodsImplementedButNotSent: SwaLint terminates with a bug
:INSTANCE-LOC:                31
:CLASS-LOC:                   0
:MCZ:                         PVpubTests-AnalogClockPrototype05-pv.1
:MCZ-TIME:                    Time: 5 December 2013, 9:45:38.638 pm
:END:
**** VER006
Add current class comment
***** Implemented Requirements
implement [[#REQ111]]
***** Metrics PVpub-AnalogClockPrototype05
:PROPERTIES:
:SWA-LINT_MethodsImplementedButNotSent: 1
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                139
:CLASS-LOC:                   11
:MCZ:                         PVpub-AnalogClockPrototype05-pv.6
:MCZ-TIME:                    Time: 5 December 2013, 9:59:41.815 pm
:END:
***** Metrics PVpubTests-AnalogClockPrototype05
same as for VER005
**** VER007
A first draft for [[REQ105]]
***** Implemented Requirements
implement [[#REQ105]]
***** Metrics PVpub-AnalogClockPrototype05
:PROPERTIES:
:MCZ:                         PVpub-AnalogClockPrototype05-pv.7
:MCZ-TIME:                    Time: 8 December 2013, 10:15:03.414 am
:END:
***** Metrics PVpubTests-AnalogClockPrototype05
same as for VER005
**** VER008
***** Implemented Requirements
Fix [[#BUG101]]
***** Metrics PVpub-AnalogClockPrototype05
:PROPERTIES:
:CUSTOM_ID:                   VER-005-008
:MCZ:                         PVpub-AnalogClockPrototype05-pv.8
:MCZ-TIME:                    Time: 26 December 2013, 1:54:15.664 am 
:END:
***** Metrics PVpubTests-AnalogClockPrototype05
same as for VER005
*** PVpub-AnalogClockPrototype06
**** VER001
Design modification with respect to PVpub-AnalogClockPrototype05
***** Implemented Requirements
Comparable to those of PVpub-AnalogClockPrototype04
***** Implemented Design Decision
[[#DES004]]
***** Metrics
:PROPERTIES:
:CUSTOM_ID:                   VER-006-001
:SWA-LINT_VariableReferencedInOnly: 2
:SWA-LINT_ClassNotReferenced: 2
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                69
:CLASS-LOC:                   6
:MCZ:                         PVpub-AnalogClockPrototype06-pv.1
:MCZ-TIME:                    27 December 2013, 6:01:36.802 pm
:END:
**** VER002
Minor changes to improve readability
***** Implemented Requirements
[[REQ113]]
***** Metrics
:PROPERTIES:
:CUSTOM_ID:                   VER-006-002
:SWA-LINT_DebuggingCodeLeftInMethods: 2
:SWA-LINT_ClassNotReferenced: 5
:SWA-Lint_InconsistentMethodClassification: 2
:INSTANCE-LOC:                94
:CLASS-LOC:                   2
:MCZ:                         PVpub-AnalogClockPrototype06-pv.2
:MCZ-TIME:                    02 January 2014, 
:END:
** Bugs
*** 
**** BUG101 - Extension for CircularLayout is missing
:PROPERTIES:
:CUSTOM_ID: BUG101
:PREPARE: Someone who wants to install Dial05 will run into problems. He needs several extensions.
These extensions 
:BUG: are not 
:EXPECTED: should be
:END: part of the PVpub-AnalogClockPrototype05 package.
:FOUND: 2013-12-08
:SOLVED: 2013-12-26
:END:
**** BUG102 - Resize is not doing well when embedding a clock into another
:PROPERTIES:
:CUSTOM_ID: BUG102
:PREPARE: Embed one clock into another with "embed into". Resize. 
The embedded clock
:BUG: is not 
:EXPECTED: is
:END: aligned with the embedding one.
:FOUND: 2013-12-26
:SOLVED: 2013-12-
:END:
**** BUG103 - use CircularLayout to embed one clock in another
:PROPERTIES:
:CUSTOM_ID: BUG103
:PREPARE: Look at the design. The layout between a clock and another embedded into it is done via
:BUG: clumsy computations in drawOn: 
:EXPECTED: CircularLayout
:END: .
:FOUND: 2013-12-27
:SOLVED: 2013-12-
:END:
**** BUG104 - Show UML diagrams as part of this text
:PROPERTIES:
:CUSTOM_ID: BUG104
:PREPARE: Read this text, especially the section "Design Decisions". It
:BUG: is not 
:EXPECTED: is
:END: possible to show simple UML diagrams here which can be exported to LaTeX & DocBook.
:FOUND: 2013-12-27
:SOLVED: 2013-12-
:END:
**** BUG105 - Avoid redraw of watchhand which could have been left unchanged with slightly different vertices
:PROPERTIES:
:CUSTOM_ID: BUG105
:PREPARE: Let the clock do one second-step. Look at the minutes watchhand or the hours watchhand.
Try different clock sizes. You will find  
:BUG: a size where intended changes on the second watchhand
lead to unintended slight position changes 
:Expected: no size where intended changes on the second watchhand
lead to unintended slight position changes
:END: on the minutes or hours watchhand.
:FOUND: 
:SOLVED: 2013-12-
:END:
* Appendix
** Code snippets
*** Compute Instance-LOC and Class-LOC
#+NAME: Compute Instance-LOC and Class-LOC
#+BEGIN_SRC Smalltalk
| instanceLoc classLoc |
instanceLoc := classLoc := 0.
(PackageInfo named: 'PVpubTests-AnalogClockPrototype05') classes
	do: [:class | 
		instanceLoc := class linesOfCode + instanceLoc.
		classLoc := class class linesOfCode + classLoc].
Transcript show: 'instanceLoc=' , instanceLoc asString , ', classLoc=' , classLoc asString.
#+END_SRC


!

!Dial06 methodsFor: 'drawing' stamp: 'pv 1/2/2014 14:49'!
bigEnough
	self flag: #REQ110.
	^ self extent > (60 @ 60)! !

!Dial06 methodsFor: 'drawing' stamp: 'pv 12/28/2013 13:31'!
drawOn: aCanvas 
	self flag: #REQ109.
	self extentHasChanged
		ifTrue: [watchhands
				do: [:each | each visible: self bigEnough; step]].
	super drawOn: aCanvas! !

!Dial06 methodsFor: 'drawing' stamp: 'pv 12/27/2013 18:45'!
extentHasChanged
	| currentExtent oldExtent change |
	currentExtent := self bounds corner - self bounds origin.
	oldExtent := boundsLag corner - boundsLag origin.
	change := currentExtent ~= oldExtent.
	change
		ifTrue: [boundsLag := self bounds].
	^change.! !

!Dial06 methodsFor: 'initialization' stamp: 'pv 1/2/2014 14:45'!
initialize
	super initialize.
	self extent: 200 @ 200;
		 position: 300 @ 300;
		 layoutPolicy: ConcentricLayout new.
	"nearest to the dial comes first."
	self pvtAddWatchhands.
	boundsLag := 1 @ 1 corner: 2 @ 2! !

!Dial06 methodsFor: 'initialization' stamp: 'pv 1/2/2014 15:37'!
pvtAddWatchhands
	watchhands := Dictionary newFrom: {#hour -> Watchhand06 new . #min -> Watchhand06 new . #sec -> Watchhand06 new}.
	(watchhands at: #hour)
		setInitialMidnightVerticesFrom: self
		withLozengeFractionX: 0.15
		andY: 0.6;
		 setTimeMessage: #hours divisor: 12.
	(watchhands at: #min)
		setInitialMidnightVerticesFrom: self
		withLozengeFractionX: 0.08
		andY: 0.85;
		 setTimeMessage: #minutes divisor: 60.
	(watchhands at: #sec)
		setInitialMidnightVerticesFrom: self
		withLozengeFractionX: 0.05
		andY: 0.9;
		 setTimeMessage: #seconds divisor: 60! !

PolygonMorph subclass: #Watchhand06
	instanceVariableNames: 'watchhand06Midnight myTimeMessage divisor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PVpub-AnalogClockPrototype06'!

!Watchhand06 commentStamp: 'pv 12/27/2013 20:46' prior: 0!
A Watchhand06 is a resizeable watchhand. It has a member watchhand06Midnight which represents an invisible, non-rotating watchhand in midnight position.

Instance Variables
!

!Watchhand06 class methodsFor: 'geometry' stamp: 'pv 12/28/2013 13:22'!
inDegrees: aTime withDivisor: divisor
	^ 360 * aTime asFloat / divisor! !

!Watchhand06 methodsFor: 'initialization' stamp: 'pv 12/27/2013 18:18'!
initialize
	super initialize.
	watchhand06Midnight := PolygonMorph new! !

!Watchhand06 methodsFor: 'geometry' stamp: 'pv 12/27/2013 14:50'!
rotationDegreesFromMidnightPosition: degrees 
	| flex center |
	(center := self valueOfProperty: #referencePosition)
		ifNil: [self setProperty: #referencePosition toValue: (center := self bounds center)].
	flex := (MorphicTransform offset: center negated)
				withAngle: degrees  degreesToRadians.
	self
		setVertices: (vertices
				collect: [:v | (flex transform: v)
						- flex offset]).
	self forwardDirection: degrees! !

!Watchhand06 methodsFor: 'initialization' stamp: 'pv 1/2/2014 15:42'!
setInitialMidnightVerticesFrom: aCircleMorph withLozengeFractionX: aFractionFloatX andY: aFractionFloatY 
	"2013-12-29pv Rautendiagonalen schneiden sich im
	Kreismittelpunkt "
	"2013-09-29pv: vertices sind immer Float"
	| lozengeLengthFromCenter lozengeWidthFromCenter radius |
	radius := aCircleMorph extent x / 2.
	lozengeLengthFromCenter := 3 max: (radius * aFractionFloatY) rounded.
	lozengeWidthFromCenter := 2 max: (radius * aFractionFloatX) rounded.
	watchhand06Midnight
		setVertices: (Array
				with: aCircleMorph center + (0 @ lozengeWidthFromCenter)
				with: aCircleMorph center + (-1 * lozengeWidthFromCenter @ 0)
				with: aCircleMorph center + (0 @ -1 * lozengeLengthFromCenter)
				with: aCircleMorph center + (lozengeWidthFromCenter @ 0)).
	aCircleMorph
		addMorph: watchhand06Midnight
		fullFrame: (LayoutConcentricFrame
				left: aFractionFloatX
				right: aFractionFloatX
				top: aFractionFloatY
				bottom: aFractionFloatX).
	watchhand06Midnight visible: false.
	aCircleMorph addMorph: self! !

!Watchhand06 methodsFor: 'initialization' stamp: 'pv 12/28/2013 13:40'!
setTimeMessage: constString divisor: anInteger
	myTimeMessage := constString.
	"what message to send to the receiver Time"
	divisor := anInteger.
	"in how many parts do we divide a full circle for this time
	object attribute"! !

!Watchhand06 methodsFor: 'stepping and presenter' stamp: 'pv 12/28/2013 13:22'!
step
	vertices := watchhand06Midnight vertices.
	self setRotationCenterFrom: owner center.
	self
		rotationDegreesFromMidnightPosition: (self class inDegrees: (DateAndTime now perform: myTimeMessage) withDivisor: divisor)! !

!Watchhand06 methodsFor: 'stepping and presenter' stamp: 'pv 12/27/2013 14:50'!
stepTime
	^ 1000! !
